"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const HttpError_1 = require("../HttpError");
/** Error Routing
 *
 * Seperates Error Handlers based on status codes
 * Falls back to a default handler if one is set.
 */
class ErrorRouting extends Map {
    /** Format Error
     *
     * @param {any} error
     * @returns {InternalError}
     */
    static formatError(error) {
        switch (typeof error) {
            case "bigint":
                error = Number(error);
            case "number":
                return {
                    status: error,
                    message: (0, HttpError_1.getMessage)(error) || "An unknown Error occured!"
                };
            case "string":
                return {
                    message: error
                };
            default:
                return {
                    status: error.code || error.status || error.statusCode,
                    message: error.message || String(error)
                };
        }
    }
    /** Internal Handle Error
     *
     * Performs error handler if it exists
     *
     * If handler throws new error, will double check if error handler can handle that error.
     *
     * @param {InternalError} error
     * @param {Context} context
     * @returns {Promise<InternalError>}
     */
    async _handle(error, context) {
        if (error.status && this.has(error.status)) {
            try {
                await this.get(error.status)(error.message, context);
            }
            catch (e) {
                return this._handle(ErrorRouting.formatError(e), context);
            }
        }
        return error;
    }
    /** Handle Error
     *
     * Will check for handlers based on error status.
     *
     * If not no response is sent, will check for defualt error handler.
     *
     * @param {any} error
     * @param {Context} context
     */
    async handle(error, context) {
        let { status, message } = await this._handle(ErrorRouting.formatError(error), context);
        if (!context.response.commited() && this._default) {
            this._default({
                status: status || "",
                message
            }, context);
        }
    }
    set() {
        switch (typeof arguments[0]) {
            case "function":
                this._default = arguments[0];
                break;
            case "string":
            case "number":
                if (typeof arguments[1] !== "function")
                    throw new TypeError("Error Handler must be a function!");
                super.set(arguments[0], arguments[1]);
                break;
            default:
                throw new TypeError("Error status must be a number or string!");
        }
        return this;
    }
}
exports.default = ErrorRouting;
